<!-- -*-visual-line-*- -->

[[Previous]](advanced-uow.md)
[[Up]](dusty-corners.md)
[[Next]](contact-us.md)

## Software Version Tracking

One of the key features of data accountability is knowledge of what version of your software (and our software!) was used to process any given task. 

When Ziggy starts a pipeline task, it automatically puts version information into the task's database entry, and the version is automatically updated any time the task is restarted. This is done by reading in properties from the `ziggy-build.properties` file in Ziggy's `build/etc` directory. Here's an example of what that file looks like:

```
# This file is automatically generated by Ziggy.
# Do not edit.
ziggy.version = Ziggy-0.6.0-20240726-323-g75138d913c
ziggy.version.branch = feature/ZIGGY-430-pipeline-version
ziggy.version.commit = 75138d913c
```

The `ziggy-build.properties` file, in turn, is constructed at the end of Ziggy's build. There's a Java program that runs a few Git commands, captures the output, and uses them to construct the build properties file. Whenever a pipeline task starts or restarts, it reads the file and stores the content of `ziggy-build.properties`, obtains the value of `ziggy.version`, and stores it in the database. 

Now, this is all well and good, but you are undoubtedly more interested in the version of the pipeline software that was used for a given task! After all, it's the pipeline software that contains the algorithms, and the pipeline results are going to be far more sensitive to changes in the algorithms than changes in Ziggy. Fortunately, Ziggy provides a method to perform a similar capture of Git information for a repository of pipeline files. 

To make this work, you need to first ensure that the `ziggy.pipeline.classpath` in the pipeline properties file includes the entry `${ziggy.home.dir}/libs/*`  (for a gentle refresher on the properties files, take a look at [the article on configuring a pipeline](configuring-pipeline.md), and [the appendix article on properties files](properties.md)). Once you've done that, you can use the Ziggy command `$ZIGGY_HOME/bin/ziggy generate-build-info`, which will generate the file `pipeline-build.properties`. The resulting file will go in an `etc` subdirectory of whatever directory you set up as the pipeline's home directory (i.e., the directory that property `ziggy.pipeline.home.dir` points to). When Ziggy runs a pipeline task, it will automatically pick up the pipeline version from this file and store it in the database along with the Ziggy version. 

We recommend that you include this command that generates the pipeline build file in your build system. All build systems we're familiar with have an option for a build target / task that runs a shell command; you can use that capability to run the command above. 

Note that this tool will only work with Git repositories. If you're using something other than Git, and would like to track your software versions, [contact us](contact-us.md) and we'll put one together for you. 

Alternately, you can write a step into your own build system that generates a `pipeline-build.properties` file when you perform your build. As described above, the output of this build step must be put into the `etc` subdirectory of the directory specified by the `ziggy.pipeline.home.dir` property. The `pipeline-build.properties` file must contain a line that defines property `pipeline.version`; you can set this equal to any text string that makes sense, based on your version control system. You can also set the properties `pipeline.version.branch` and `pipeline.version.commit` if you have values for these that make sense, or you can leave them out. 

[[Previous]](advanced-uow.md)
[[Up]](dusty-corners.md)
[[Next]](contact-us.md)

